aboutsummaryrefslogtreecommitdiff
path: root/gas
diff options
context:
space:
mode:
authorKen Raeburn <raeburn@cygnus>1994-09-28 19:09:16 +0000
committerKen Raeburn <raeburn@cygnus>1994-09-28 19:09:16 +0000
commit648acaa0c0427463876773336c87af4d18be8a76 (patch)
tree924b9442deba3959f8d22dfc90a63e9dc9e01e0f /gas
parent67b9f39403addf64801bc97a66e34a5dd2a93252 (diff)
downloadfsf-binutils-gdb-648acaa0c0427463876773336c87af4d18be8a76.zip
fsf-binutils-gdb-648acaa0c0427463876773336c87af4d18be8a76.tar.gz
fsf-binutils-gdb-648acaa0c0427463876773336c87af4d18be8a76.tar.bz2
* expr.c (make_expr_symbol): No longer static. Use symbol_create, not
symbol_new, for symbols holding expression values. * expr.h (make_expr_symbol): Move declaration here.
Diffstat (limited to 'gas')
-rw-r--r--gas/ChangeLog6
-rw-r--r--gas/expr.h106
2 files changed, 85 insertions, 27 deletions
diff --git a/gas/ChangeLog b/gas/ChangeLog
index 50df216..efca056 100644
--- a/gas/ChangeLog
+++ b/gas/ChangeLog
@@ -1,3 +1,9 @@
+Wed Sep 28 14:56:39 1994 Ken Raeburn <raeburn@cujo.cygnus.com>
+
+ * expr.c (make_expr_symbol): No longer static. Use symbol_create,
+ not symbol_new, for symbols holding expression values.
+ * expr.h (make_expr_symbol): Move declaration here.
+
Wed Sep 28 11:50:40 1994 Ian Lance Taylor (ian@sanguine.cygnus.com)
* config/tc-mips.c: Include subsegs.h.
diff --git a/gas/expr.h b/gas/expr.h
index d98b486..c7b97da 100644
--- a/gas/expr.h
+++ b/gas/expr.h
@@ -1,5 +1,5 @@
/* expr.h -> header file for expr.c
- Copyright (C) 1987, 1992 Free Software Foundation, Inc.
+ Copyright (C) 1987, 1992, 1993 Free Software Foundation, Inc.
This file is part of GAS, the GNU Assembler.
@@ -18,43 +18,94 @@
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
/*
- * Abbreviations (mnemonics).
- *
- * O operator
- * Q quantity, operand
- * X eXpression
- */
-
-/*
* By popular demand, we define a struct to represent an expression.
* This will no doubt mutate as expressions become baroque.
*
* Currently, we support expressions like "foo-bar+42".
* In other words we permit a (possibly undefined) minuend, a
* (possibly undefined) subtrahend and an (absolute) augend.
* RMS says this is so we can have 1-pass assembly for any compiler
* emissions, and a 'case' statement might emit 'undefined1 - undefined2'.
*
- * To simplify table-driven dispatch, we also have a "segment" for the
- * entire expression. That way we don't require complex reasoning about
- * whether particular components are defined; and we can change component
- * semantics without re-working all the dispatch tables in the assembler.
- * In other words the "type" of an expression is its segment.
+ * The type of an expression used to be stored as a segment. That got
+ * confusing because it overloaded the concept of a segment. I added
+ * an operator field, instead.
*/
-typedef struct
+/* This is the type of an expression. The operator types are also
+ used while parsing an expression.
+
+ NOTE: This enumeration must match the op_rank array in expr.c. */
+
+typedef enum
+{
+ /* An illegal expression. */
+ O_illegal,
+ /* A nonexistent expression. */
+ O_absent,
+ /* X_add_number (a constant expression). */
+ O_constant,
+ /* X_add_symbol + X_add_number. */
+ O_symbol,
+ /* A register (X_add_number is register number). */
+ O_register,
+ /* A big value. If X_add_number is negative or 0, the value is in
+ generic_floating_point_number. Otherwise the value is in
+ generic_bignum, and X_add_number is the number of LITTLENUMs in
+ the value. */
+ O_big,
+ /* (- X_add_symbol) + X_add_number. */
+ O_uminus,
+ /* (~ X_add_symbol) + X_add_number. */
+ O_bit_not,
+ /* (X_add_symbol * X_op_symbol) + X_add_number. */
+ O_multiply,
+ /* (X_add_symbol / X_op_symbol) + X_add_number. */
+ O_divide,
+ /* X_add_symbol % X_op_symbol) + X_add_number. */
+ O_modulus,
+ /* X_add_symbol << X_op_symbol) + X_add_number. */
+ O_left_shift,
+ /* X_add_symbol >> X_op_symbol) + X_add_number. */
+ O_right_shift,
+ /* X_add_symbol | X_op_symbol) + X_add_number. */
+ O_bit_inclusive_or,
+ /* X_add_symbol |~ X_op_symbol) + X_add_number. */
+ O_bit_or_not,
+ /* X_add_symbol ^ X_op_symbol) + X_add_number. */
+ O_bit_exclusive_or,
+ /* X_add_symbol & X_op_symbol) + X_add_number. */
+ O_bit_and,
+ /* X_add_symbol + X_op_symbol) + X_add_number. */
+ O_add,
+ /* X_add_symbol - X_op_symbol) + X_add_number. */
+ O_subtract
+} operatorT;
+
+typedef struct expressionS
{
- struct symbol *X_add_symbol; /* "foo", above */
- struct symbol *X_subtract_symbol; /* "bar", above */
- offsetT X_add_number; /* 42, above -- must be signed */
- /* What segment (expr type)? */
- segT X_seg;
+ /* The type of the expression. */
+ operatorT X_op;
+ /* The main symbol. */
+ struct symbol *X_add_symbol;
+ /* The second symbol, if needed. */
+ struct symbol *X_op_symbol;
+ /* A number to add. */
+ offsetT X_add_number;
+ /* Non-zero if X_add_number should be regarded as unsigned. This is
+ only valid for O_constant expressions. It is only used when an
+ O_constant must be extended into a bignum (i.e., it is not used
+ when performing arithmetic on these values).
+ FIXME: This field is not set very reliably.
+ If we ever need more flags here, we can make them bitfields. */
+ int X_unsigned;
} expressionS;
/* "result" should be type (expressionS *). */
#define expression(result) expr(0,result)
/* If an expression is SEG_BIG, look here for its value. These common
data may be clobbered whenever expr() is called. */
/* Flonums returned here. Big enough to hold most precise flonum. */
extern FLONUM_TYPE generic_floating_point_number;
@@ -68,5 +119,6 @@ typedef char operator_rankT;
char get_symbol_end PARAMS ((void));
segT expr PARAMS ((int rank, expressionS * resultP));
unsigned int get_single_number PARAMS ((void));
+symbolS *make_expr_symbol PARAMS ((expressionS * expressionP));
/* end of expr.h */
@@ -18,43 +18,94 @@
* This will no doubt mutate as expressions become baroque.
*
- * Currently, we support expressions like "foo-bar+42".
- * In other words we permit a (possibly undefined) minuend, a
- * (possibly undefined) subtrahend and an (absolute) augend.
- * RMS says this is so we can have 1-pass assembly for any compiler
- * emissions, and a 'case' statement might emit 'undefined1 - undefined2'.
+ * Currently, we support expressions like "foo OP bar + 42". In other
+ * words we permit a (possibly undefined) symbol, a (possibly
+ * undefined) symbol and the operation used to combine the symbols,
+ * and an (absolute) augend. RMS says this is so we can have 1-pass
+ * assembly for any compiler emissions, and a 'case' statement might
+ * emit 'undefined1 - undefined2'.
*
@@ -18,43 +18,94 @@
} expressionS;
/* "result" should be type (expressionS *). */
-#define expression(result) expr(0,result)
+#define expression(result) expr (0, result)
-/* If an expression is SEG_BIG, look here for its value. These common
+/* If an expression is O_big, look here for its value. These common
data may be clobbered whenever expr() is called. */
/* Flonums returned here. Big enough to hold most precise flonum. */